/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.jobnet.dispatcher;
import java.util.HashMap;

import org.apache.log4j.Logger;

import com.sos.JSHelper.Exceptions.JobSchedulerException;
import com.sos.jobnet.interfaces.IDispatchHandler;
import com.sos.jobnet.interfaces.IJobNetCallback;
import com.sos.jobnet.interfaces.IJobNetCollection;
import com.sos.jobnet.interfaces.IPlanItem;
import com.sos.jobnet.options.JobNetOptions;

/**
* \class JobNetDispatcher 
* 
* \brief JobNetDispatcher - 
* 
* \details
*
* \section JobNetDispatcher.java_intro_sec Introduction
*
* \section JobNetDispatcher.java_samples Some Samples
*
* \code
*   .... code goes here ...
* \endcode
*
* <p style="text-align:center">
* <br />---------------------------------------------------------------------------
* <br /> APL/Software GmbH - Berlin
* <br />##### generated by ClaviusXPress (http://www.sos-berlin.com) #########
* <br />---------------------------------------------------------------------------
* </p>
* \author Uwe Risse
* @version $Id: JobNetDispatcher.java 16680 2012-03-01 12:43:54Z ur $
* \see reference
*
* Created on 24.02.2012 13:02:19
 */
/**
 * @author Uwe Risse
 *
 */
public class JobNetDispatcher implements IJobNetCallback{
	@SuppressWarnings("unused")

	public  final String	conSVNVersion	= "$Id: JobNetDispatcher.java 16680 2012-03-01 12:43:54Z ur $";
	private final String		conClassName		= "JobNetDispatcher";
	private static final Logger	logger				= Logger.getLogger(JobNetDispatcher.class);
	private IDispatchHandler	objDispatchHandler	= null;
	private IJobNetCallback 	objJobNetCallback	= this;
	

	private JobNetOptions		objOptions			= null;


	public JobNetDispatcher() {
		//
	}
	
	public void setIJobNetCallback(IJobNetCallback pobjJobNetCallback) {
		if (pobjJobNetCallback == null) {
			objJobNetCallback = this;
		}
		else {
			objJobNetCallback = pobjJobNetCallback;
		}	
	}

	public JobNetOptions Options() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::Options";
		if (objOptions == null) {
			objOptions = new JobNetOptions();
		}
		return objOptions;
	} // public JobNetOptions Options

	public void setOptions(JobNetOptions objOptions) {
		this.objOptions = objOptions;
	}
	
	public void setOptionsUsingHashMap(HashMap<String, String> objHshMap) {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::setOptionsUsingHashMap";
		try {
			Options().setAllOptions(objHshMap);
		}
		catch (Exception e) {
 			e.printStackTrace();
		}
	} // public void setOptionsUsingHashMap

	public void doInit() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::doInit";
		objDispatchHandler = new JobNetDBDispatcher(objOptions);
	} // public void doInit

	public JobNetDispatcher run() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::run";
		objDispatchHandler.open();
		
 		try {
			objDispatchHandler.setDispatcherStatusToRunning();
			objDispatchHandler.setNodeStatusToDispatching();
			if (!objDispatchHandler.isRunnerSkipped()){
				if (objJobNetCallback.getExitCode()==0){
				      objDispatchHandler.setRunnerStatusToFinished();
				      dispatch();
				}else{
				      objDispatchHandler.setRunnerStatusToError();
				}
			}else{
			      dispatch();
			}
			
 			 
			objDispatchHandler.setNodeStatusToFinished();
			objDispatchHandler.setDispatcherStatusToFinished();
		}
		catch (Exception e) {
			objDispatchHandler.setDispatcherStatusToError();
			objDispatchHandler.setNodeStatusToError();
			throw new JobSchedulerException("Exception raised", e);
		}
		objDispatchHandler.close();
		return this;
	} // public JobNetDispatcher run
	
	private void sleep(int sleepTime){
 		 
		try {
			if (sleepTime > 0){
			   logger.debug(String.format("Dispatcher is going to sleep for %1s seconds",sleepTime));	
			   Thread.sleep(sleepTime*1000);
			}
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}
	private void dispatch(){
 		
		sleep(objOptions.delay_start_dispatcher.getTimeAsSeconds());
		IJobNetCollection jobNetCollection = objDispatchHandler.getDispatchingNodes();
		for(int i = 0; i < jobNetCollection.size();i++){
			IPlanItem planItem = jobNetCollection.get(i);
			objJobNetCallback.startNode(planItem);
 		}
	}
	


	@Override
	public int getExitCode() {
		return 0;
	}

	@Override
	public void setStateText(String message) {
		
	}

	@Override
	public void startNode(IPlanItem planItem) {
		logger.debug(String.format("Starting Node: %1s",planItem.getPlanId()));
		logger.debug(String.format("  Node: %1s",planItem.getJobnetNodeItem().getNode()));
		logger.debug(String.format("  in Net: %1s",planItem.getJobnetNodeItem().getJobnet()));
		logger.debug(String.format("  XML: %1s",planItem.getOrderXml()));
		
	}

	@Override
	public void skipRunner() {
 		
	}

	 
}
