/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package sos.mail;
import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.util.regex.Matcher;

import javax.mail.Flags;
import javax.mail.Folder;
import javax.mail.Message;
import javax.mail.search.SubjectTerm;

import sos.mail.options.SOSMailProcessInboxOptions;
import sos.net.SOSMailReceiver;
import sos.net.SOSMimeMessage;
import sos.scheduler.command.SOSSchedulerCommand;
import sos.scheduler.job.JobSchedulerJobAdapter;
import sos.spooler.Job_chain;
import sos.spooler.Order;
import sos.spooler.Variable_set;
import sos.util.SOSSchedulerLogger;

import com.sos.JSHelper.Exceptions.JobSchedulerException;

/**
 * 
 */
public class SOSMailProcessInbox extends JobSchedulerJobAdapter {
	
	private final String		conClassName				= "SOSMailProcessInbox";  //$NON-NLS-1$

	private SOSMailReceiver		receiver;
	private SOSSchedulerLogger	sosLogger					= null;
	private final SOSMailProcessInboxOptions objO 			= null;
	private boolean				isLocalScheduler			= true;
	
	
	@Override
	public boolean spooler_process() throws Exception {
		try {
			SOSMailProcessInboxOptions objO = new SOSMailProcessInboxOptions();
			sosLogger = new SOSSchedulerLogger(spooler_log);
			objO.setAllOptions(getSchedulerParameterAsProperties(getJobOrOrderParameters()));
			if (!objO.mail_scheduler_host.isDirty()) {
				objO.mail_scheduler_host.Value(spooler.hostname());
			}
			if (!objO.mail_scheduler_port.isDirty()) {
				objO.mail_scheduler_port.value(spooler.tcp_port());
			}
			if(objO.mail_use_seen.value()) {
				objO.mail_set_seen.value(true);
			}
			objO.CheckMandatory();
			sosLogger.info(objO.dirtyString());
//			sosLogger.info(objO.toString());
			isLocalScheduler = objO.mail_scheduler_host.Value().equals(spooler.hostname()) && objO.mail_scheduler_port.value() == spooler.tcp_port();
			
			ArrayList<SOSMimeMessage> messages = findMessages();
			if (messages == null) {
				return isOrderJob();
			}
			if (!messages.isEmpty()) {
				Iterator<SOSMimeMessage> messageIterator = messages.iterator();
				SOSMimeMessage newestMail = null;
				while (messageIterator.hasNext()) {
					SOSMimeMessage message = messageIterator.next();
					sosLogger.info(message.getSubject());
					Date messageDate = message.getSentDate();
					if (newestMail == null || messageDate.after(newestMail.getSentDate()))
						newestMail = message;
					performAction(message);
				}
			}
			if (receiver != null)
				receiver.closeFolder(true);
		}
		catch (Exception e) {
			String stateText = e.toString();
			spooler_job.set_state_text(stateText);
			sosLogger.info("Job " + spooler_job.name() + " terminated with errors.");
			sosLogger.error("an error occurred while processing: " + stateText);
			throw new JobSchedulerException("", e);
		}
		
		return isOrderJob();
	}

	private void performAction(final SOSMimeMessage message) throws Exception {
		String action = "";
		StringTokenizer t = new StringTokenizer(objO.mail_action.Value(), ",");
		while (t.hasMoreTokens()) {
			action = t.nextToken();
			sosLogger.debug3("Action " + action + " will be performed.");
			action = action.toLowerCase();
			if (action.equals("dump"))
				dumpMessage(message);
			if (action.equals("order"))
				startOrder(message);
			if (action.equals("command"))
				executeCommand(message);
			if (action.equals("delete"))
				deleteMessage(message);
			if (objO.mail_set_seen.value())
				message.setFlag(Flags.Flag.SEEN, true);
		}
	}

	private ArrayList <SOSMimeMessage> findMessages() throws Exception {
		ArrayList <SOSMimeMessage> messages = new ArrayList <SOSMimeMessage> ();
		try {
			sosLogger.debug3(String.format("Connecting to Mailserver %1$s:%2$d(%3$s)...",objO.mail_host.Value(),objO.mail_port.value(),objO.mail_server_type.Value()));
			receiver = new SOSMailReceiver(objO.mail_host.Value(), objO.mail_port.Value(), objO.mail_user.Value(), objO.mailPassword.Value());
			receiver.setLogger(sosLogger);
			receiver.connect(objO.mail_server_type.Value());
			if (objO.mail_server_timeout.value() > 0) {
				receiver.setTimeout(objO.mail_server_timeout.value());
			}
			sosLogger.debug3("reading " + objO.mail_message_folder.Value());
			Folder folder = receiver.openFolder(objO.mail_message_folder.Value(), receiver.READ_WRITE);
			int max = folder.getMessageCount();
			SubjectTerm term = null;
			Message[] msgs = null;
			Message[] msgs2 = null;
			term = new SubjectTerm(objO.mail_subject_filter.Value());
			msgs = folder.getMessages(1, max);
			if (objO.mail_subject_filter.IsNotEmpty()) {
				sosLogger.debug3(String.format("looking for %1$s", objO.mail_subject_filter.Value()));
				msgs2 = folder.search(term, msgs);
				sosLogger.debug3(String.format("%1$s messages found with %2$s", msgs2.length, objO.mail_subject_filter.Value()));
			}
			else {
				msgs2 = msgs;
				sosLogger.debug3(msgs2.length + " messages found");
			}
			if (msgs2.length > 0) {
				for (Message element : msgs2) {
					if (objO.mail_use_seen.value() && element.isSet(Flags.Flag.SEEN)) {
						sosLogger.info("message skipped, already seen: " + element.getSubject());
						continue;
					}
					SOSMimeMessage message = new SOSMimeMessage(element, sosLogger);
					// skip mails that do not match the subject pattern
					if (objO.mail_subject_pattern.IsNotEmpty()) {
						objO.mail_subject_pattern.setRegExpFlags(0);
						Matcher subjectMatcher = objO.mail_subject_pattern.getPattern().matcher(message.getSubject());
						if (!subjectMatcher.find()) {
							sosLogger.info(String.format("message skipped, subject does not match [%1$s]: %2$s", objO.mail_subject_pattern.Value(), message.getSubject()));
							continue;
						}
					}
					// skip mails whose body does not match the download link pattern
					if (objO.mail_body_pattern.IsNotEmpty()) {
						objO.mail_body_pattern.setRegExpFlags(0);
						Matcher bodyMatcher = objO.mail_body_pattern.getPattern().matcher(message.getPlainTextBody());
						if (!bodyMatcher.find()) {
							sosLogger.info(String.format("message skipped, body does not match [%1$s]: %2$s", objO.mail_body_pattern.Value(), message.getPlainTextBody()));
							continue;
						}
					}
					messages.add(message);
				}
			}
		}
		catch (Exception e) {
			e.printStackTrace();
			sosLogger.error(e.toString());
			throw new Exception("Error occured querying mail server. " + e);
		}
		return messages;
	}

	private void executeCommand(final SOSMimeMessage message) throws Exception {
		if (isLocalScheduler) {
			sosLogger.debug3("...host/port is this host and port. Using API");
			spooler.execute_xml(message.getPlainTextBody());
		}
		else {
			executeXml(message.getPlainTextBody());
		}
	}

	private void dumpMessage(final SOSMimeMessage message) throws Exception {
		if (objO.mail_dump_dir.IsNotEmpty()) {
			throw new Exception("No output directory specified.");
		}
		File messageFile = new File(objO.mail_dump_dir.Value(), message.getMessageId());
		sosLogger.debug3("saving message to file: " + messageFile.getAbsolutePath());
		message.dumpMessageToFile(messageFile, true, false);
	}

	private void deleteMessage(final SOSMimeMessage message) throws Exception {
		sosLogger.debug3("deleting message : " + message.getSubject());
		message.deleteMessage();
	}

	private void startOrder(final SOSMimeMessage message) throws Exception {
		String jobchain = objO.mail_jobchain.Value();
		Variable_set order_params = spooler.create_variable_set();
		sosLogger.debug3("....merge");
		order_params.merge(spooler_task.params());
		order_params.set_var("mail_from", message.getFrom());
		if (message.getFromName() != null) {
			order_params.set_var("mail_from_name", message.getFromName());
		}
		else {
			order_params.set_var("mail_from_name", "");
		}
		order_params.set_var("mail_message_id", message.getMessageId());
		order_params.set_var("mail_subject", message.getSubject());
		order_params.set_var("mail_body", message.getPlainTextBody());
		order_params.set_var("mail_send_at", message.getSentDateAsString());
		if (isLocalScheduler) {
			sosLogger.debug3("...host/port is this host and port. Using API");
			Job_chain j = spooler.job_chain(jobchain);
			sosLogger.debug3("...jobchain " + jobchain + " created.");
			Order o = spooler.create_order();
			o.params().merge(order_params);
			// if (!id.equals(""))o.set_id(id);
			sosLogger.debug3("...order " + o.id() + " created.");
			if (objO.mail_order_state.IsNotEmpty())
				o.set_state(objO.mail_order_state.Value());
			if (objO.mail_order_title.IsNotEmpty())
				o.set_title(objO.mail_order_title.Value());
			j.add_order(o);
			sosLogger.debug3("...order added to " + jobchain);
		}
		else {
			startOrderXML(order_params);
		}
	}

	private void startOrderXML(final Variable_set params_) throws Exception {
		String id = objO.mail_order_id.Value();
		String state = objO.mail_order_state.Value();
		String title = objO.mail_order_title.Value();
		String jobchain = objO.mail_jobchain.Value();
		sosLogger.debug3("Starting order " + id + " at " + jobchain + " with xml-command");
		if (objO.mail_scheduler_host.IsEmpty())
			throw new Exception("Missing host while starting order.");
		if (objO.mail_order_id.IsNotEmpty())
			id = " id=\"" + id + "\"";
		if (objO.mail_order_state.IsNotEmpty())
			state = " state=\"" + state + "\"";
		if (objO.mail_order_title.IsNotEmpty())
			title = " title=\"" + title + "\"";
		String xml = "<add_order replace=\"yes\"" + id + title + state + " job_chain=\"" + jobchain + "\"><params>";
		if (params_ != null && params_.xml() != null && params_.xml().length() > 0) {
			String pparamsXml = params_.xml();
			int begin = pparamsXml.indexOf("<sos.spooler.variable_set>") + 26;
			int end = pparamsXml.lastIndexOf("</sos.spooler.variable_set>");
			if (begin >= 26 && end >= 26)
				xml += pparamsXml.substring(begin, end).replaceAll("variable", "param");
		}
		xml += "</params></add_order>";
		executeXml(xml);
	}

	private void executeXml(final String xml) throws Exception {
		SOSSchedulerCommand command;
		command = new SOSSchedulerCommand(objO.mail_scheduler_host.Value(), objO.mail_scheduler_port.value());
		command.setProtocol("udp");
		sosLogger.debug3("Trying connection to " + objO.mail_scheduler_host.Value() + ":" + objO.mail_scheduler_port.Value());
		command.connect();
		sosLogger.debug3("...connected");
		sosLogger.debug3("Sending add_order command:\n" + xml);
		command.sendRequest(xml);
	}
}
