/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: async_event.cxx 13199 2007-12-06 14:15:42Z jz $

#include "zschimmer.h"
#include "async_event.h"
#include "log.h"


using namespace std;

namespace zschimmer {

//-----------------------------------------------------------------Event_operation::Event_operation

Event_operation::Event_operation()
:
    _zero_(this+1)
{
}

//----------------------------------------------------------------Event_operation::~Event_operation
    
Event_operation::~Event_operation()
{
}

//-----------------------------------------------------------------Event_operation::async_signaled_

bool Event_operation::async_signaled_()
{
    if( Socket_event* event = async_event() )
    {
        return event->signaled();
    }

    return false;
}

//------------------------------------------------------------Event_operation::add_to_event_manager

void Event_operation::add_to_event_manager( Event_manager* event_manager )
{
    if( _event_manager )
    {
        if( _event_manager != event_manager )  throw_xc( Z_FUNCTION );
        return;
    }

    set_async_manager( event_manager );

    _event_manager = event_manager;
    _event_manager->add_event_operation( this );
}

//-------------------------------------------------------Event_operation::remove_from_event_manager

void Event_operation::remove_from_event_manager()
{
    if( _event_manager ) 
    {
        _event_manager->remove_event_operation( this );
        _event_manager = NULL;
    }

    set_async_manager( NULL );
}

//--------------------------------------------------------------------Event_manager::Event_manager

Event_manager::Event_manager()
:
    _zero_(this+1)
{
}

//--------------------------------------------------------------------Event_manager::~Event_manager

Event_manager::~Event_manager()
{
}

//---------------------------------------------------------------Event_manager::add_event_operation

void Event_manager::add_event_operation( Event_operation* op )
{
    _event_operation_list_modified = true;  // Fr async_continue(), iteriert ber _socket_operation_map.

    _event_operation_list.push_back( op );
    //if( op->_write_socket != SOCKET_ERROR )  _socket_operation_map[ op->_write_socket ] = op;
    //if( op->_read_socket  != SOCKET_ERROR )  _socket_operation_map[ op->_read_socket  ] = op;
}

//------------------------------------------------------------Event_manager::remove_event_operation

void Event_manager::remove_event_operation( Event_operation* op )
{
    _event_operation_list_modified = true;  // Fr async_continue(), iteriert ber _socket_operation_map.

    Z_FOR_EACH( Event_operation_list, _event_operation_list, o )
    {
        if( *o == op )  { _event_operation_list.erase( o );  break; }
    }
}

//-------------------------------------------------------------------------------------------------

} //namespace zschimmer
