/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: xml.h 13358 2008-01-31 11:27:36Z jz $        Joacim Zschimmer, Zschimmer GmbH, http://www.zschimmer.com

#ifndef __ZSCHIMMER_XML_H
#define __ZSCHIMMER_XML_H

#include "z_io.h"


#define DOM_FOR_EACH_ELEMENT( PARENT, ITERATOR )                                                \
                                                                                                \
    for( ::zschimmer::xml::libxml2::Simple_node_ptr ITERATOR##_node = (PARENT).firstChild();    \
         ITERATOR##_node;                                                                       \
         ITERATOR##_node = ITERATOR##_node.nextSibling() )                                      \
                                                                                                \
    if( ::zschimmer::xml::libxml2::Element_ptr ITERATOR = xml::Element_ptr( ITERATOR##_node, ::zschimmer::xml::libxml2::Element_ptr::no_xc ) )


namespace zschimmer {
namespace xml {


namespace libxml2
{
    struct Element_ptr;
}


//--------------------------------------------------------------------------------------------const

extern const char               unicode_substition_character ;
extern const char               allowed_xml_ascii_characters [ 256 ];
extern const char               allowed_xml_latin1_characters[ 256 ];

//-------------------------------------------------------------------------------------------------

inline bool                     is_valid_xml_ascii_character ( char c )                             { return allowed_xml_ascii_characters[ (unsigned char) c ] != 0; }
inline bool                     is_valid_xml_latin1_character( char c )                             { return allowed_xml_ascii_characters[ (unsigned char) c ] != 0; }

string                          encode_text                 ( const string&, bool quote_too = false );
inline string                   encode_attribute_value      ( const string& value )                 { return encode_text( value, true ); }
bool                            is_valid_xml_ascii_character( char );
bool                            is_valid_xml_latin1_character( char );
extern string                   non_xml_latin1_characters_substituted( const io::Char_sequence& );

//---------------------------------------------------------------------------------------Xml_writer

struct Xml_writer : io::Filter_writer
{
                                Xml_writer                  ( Writer* );

    void                    set_encoding                    ( const string& encoding )              { _encoding = encoding; }                                   
    void                        write_prolog                ();

    void                        begin_element               ( const string& element_name );
    void                        set_attribute               ( const string& name, const string& );
    void                        set_attribute               ( const string& name, int64 );
    void                        set_attribute_optional      ( const string& name, const string& );
    void                        end_element                 ( const string& element_name );

    void                        write                       ( const io::Char_sequence& utf8_text );
    void                        write_through               ( const io::Char_sequence& text );
    void                        flush                       ();

    void                        write_element               ( const libxml2::Element_ptr& );

  protected:
                                Xml_writer                  ();

    void                        close_tag                   ();

  private:
    bool                       _is_tag_open;
    string                     _encoding;
};

//--------------------------------------------------------------------------------Xml_string_writer

struct Xml_string_writer : Xml_writer
{
                                Xml_string_writer           ();

    void                        flush                       ();
    string                      to_string                   ();

  private:
    ptr<io::String_writer>     _string_writer;
};

//-------------------------------------------------------------------------------------------------

} //namespace xml
} //namespace zschimmer

#endif
